/**
 * Proiectarea Algoritmilor, 2013
 * Lab 9: Arbori minimi de acoperire
 *
 * @author 	Alexandru Tudorica
 * @email	tudalex@gmail.com
 */

#include "Graph.h"



Graph::Graph()
{
}

Graph::~Graph()
{
	for(uint i = 0, sz = node_count(); i < sz; ++i){
		delete Nodes[i];
	}
}

uint Graph::node_count() const
{
	return Nodes.size();
}

void Graph::insert_node(Node* node)
{
	Nodes.push_back(node);
	Edges.push_back(std::vector< std::pair<Node*, int> >());
}

std::vector< std::pair <int, int > >& Graph::get_all_edges() {
	return all_edges;
}

void Graph::insert_edge(Node* node1, Node* node2, int cost)
{
	check_node(*node1);
	Edges[node1->get_id()].push_back(std::make_pair(node2, cost));
	Edges[node2->get_id()].push_back(std::make_pair(node1, cost));
	all_edges.push_back(std::make_pair(node1->get_id(), node2->get_id()));
}

std::vector<Node*>& Graph::get_nodes()
{
	return Nodes;
}

std::vector<std::pair<Node*, int> >& Graph::get_edges(Node& node)
{
	check_node(node);
	return Edges[node.get_id()];
}


bool Graph::exists_edge_between(Node* node1, Node* node2)
{
	std::vector< std::pair<Node*, int> > edges = Edges[node1->get_id()];
	unsigned i;
	for (i = 0; i < edges.size(); i++)
		if (edges[i].first->get_id() == node2->get_id())
			return true;
	return false;
}

int Graph::get_cost_between(Node* node1, Node* node2)
{
	std::vector< std::pair<Node*, int> > edges = Edges[node1->get_id()];
	unsigned i;
	for (i = 0; i < edges.size(); i++)
		if (edges[i].first->get_id() == node2->get_id())
			return edges[i].second;
	return MAX_VALUE;
}

void Graph::check_node( Node& node ) const
{
	uint _id = node.get_id();

	if( _id > node_count() || _id < 0 ){
		std::stringstream ss; ss >> _id;
		throw InvalidQuery( "Invalid node id: " + ss.str() );
	}
}

void Graph::reset()
{
	for (unsigned int i = 0; i < Nodes.size(); i++)
		Nodes[i]->reset();
}

std::ostream& operator<< (std::ostream& out, Graph& graph ){

	out << "Print Graph :\n";

	for( uint i = 0; i < graph.Nodes.size(); ++i )
	{
		out << "Edges for " << i << " -> ";
		for( uint j = 0, sz = graph.Edges[i].size(); j < sz; ++j ){
			out << j << "  ";
		}
		out << "\n";
	}

	return out;
}



/*
 * Function to read all the test:
 *
 * Input Format:
 * N = number of nodes
 * M = number of edges
 * M lines with Nodei Nodej	costBetweenNodeiNodej	   -- list of edges
 * @param filename
 */

std::istream& operator>> (std::istream& in , Graph& graph )
{
	uint _nodes, _edges;
	in >> _nodes;

	for (uint i = 0; i < _nodes; ++i) {
		Node *new_node = new Node(i);
		graph.insert_node(new_node);
	}

	in >> _edges;
	int node1, node2, cost;

	for( uint i = 0; i < _edges; ++i ) {
		in >> node1 >> node2 >> cost;
		graph.insert_edge(graph.Nodes[node1], graph.Nodes[node2], cost);
	}

	return in;
}
